UnixSocket

Struct UnixSocket 

pub struct UnixSocket { /* private fields */ }
Available on crate feature net only.
A Unix socket that has not yet been converted to a UnixStream or UnixListener.

Implementations

Source

impl UnixSocket

pub async fn new_stream() -> Result<UnixSocket, Error>

Creates a new Unix stream socket.

pub fn local_addr(&self) -> Result<SockAddr, Error>

Returns the local address that this socket is bound to.

pub fn take_error(&self) -> Result<Option<Error>, Error>

Returns the value of the SO_ERROR option.

pub async fn bind(&self, path: impl AsRef<Path>) -> Result<(), Error>

Binds the socket to the given address.

pub async fn bind_addr(&self, addr: &SockAddr) -> Result<(), Error>

Binds the socket to the given address.

pub async fn listen(self, backlog: i32) -> Result<UnixListener, Error>

Converts the socket into a UnixListener.

backlog defines the maximum number of pending connections are queued by the operating system at any given time. Connections are removed from the queue with UnixListener::accept. When the queue is full, the operating-system will start rejecting connections.

pub async fn connect(self, path: impl AsRef<Path>) -> Result<UnixStream, Error>

Establishes a Unix connection with a peer at the specified socket address.

See UnixSocket::connect_addr for more details.

pub async fn connect_addr(self, addr: &SockAddr) -> Result<UnixStream, Error>

Establishes a Unix connection with a peer at the specified socket address.

The UnixSocket is consumed. Once the connection is established, a connected UnixStream is returned. If the connection fails, the encountered error is returned.

On Windows, the socket should be bound to an empty address before connecting.