pub struct TcpStream { /* private fields */ }net only.Expand description
A TCP stream between a local and a remote socket.
A TCP stream can either be created by connecting to an endpoint, via the
connect method, or by accepting a connection from a listener.
Examples
use std::net::SocketAddr;
use compio_io::AsyncWrite;
use compio_net::TcpStream;
// Connect to a peer
let mut stream = TcpStream::connect("127.0.0.1:8080").await.unwrap();
// Write some data.
stream.write("hello world!").await.unwrap();Implementations
Sourceimpl TcpStream
impl TcpStream
Sourcepub async fn connect(addr: impl ToSocketAddrsAsync) -> Result<TcpStream, Error>
pub async fn connect(addr: impl ToSocketAddrsAsync) -> Result<TcpStream, Error>
Opens a TCP connection to a remote host.
To configure the socket before connecting, you can use the TcpSocket
type.
Sourcepub fn from_std(stream: TcpStream) -> Result<TcpStream, Error>
pub fn from_std(stream: TcpStream) -> Result<TcpStream, Error>
Creates new TcpStream from a std::net::TcpStream.
Sourcepub fn close(self) -> impl Future<Output = Result<(), Error>>
pub fn close(self) -> impl Future<Output = Result<(), Error>>
Close the socket. If the returned future is dropped before polling, the socket won’t be closed.
As the socket is clonable, users can call close on a clone, but the
future will never complete until all clones are dropped. Some
operations may keep a strong reference to the socket, so the future
may never complete if there are pending operations.
It’s OK to drop the socket directly without calling close, but the
socket may not be closed immediately.
Sourcepub fn peer_addr(&self) -> Result<SocketAddr, Error>
pub fn peer_addr(&self) -> Result<SocketAddr, Error>
Returns the socket address of the remote peer of this TCP connection.
Sourcepub fn local_addr(&self) -> Result<SocketAddr, Error>
pub fn local_addr(&self) -> Result<SocketAddr, Error>
Returns the socket address of the local half of this TCP connection.
Sourcepub fn take_error(&self) -> Result<Option<Error>, Error>
pub fn take_error(&self) -> Result<Option<Error>, Error>
Returns the value of the SO_ERROR option.
Sourcepub fn split(&self) -> (ReadHalf<'_, TcpStream>, WriteHalf<'_, TcpStream>)
pub fn split(&self) -> (ReadHalf<'_, TcpStream>, WriteHalf<'_, TcpStream>)
Splits a TcpStream into a read half and a write half, which can be
used to read and write the stream concurrently.
This method is more efficient than
into_split, but the halves cannot
be moved into independently spawned tasks.
Sourcepub fn into_split(self) -> (TcpStream, TcpStream)
pub fn into_split(self) -> (TcpStream, TcpStream)
Sourcepub async fn disconnect(self) -> Result<TcpSocket, Error>
Available on Windows only.
pub async fn disconnect(self) -> Result<TcpSocket, Error>
Close the connection of the socket, and reuse it to create a new
connection. This method is useful when the socket is created by
TcpListener::accept, and will be reused in
TcpListener::accept_with to accept a new connection.
Sourcepub fn nodelay(&self) -> Result<bool, Error>
pub fn nodelay(&self) -> Result<bool, Error>
Gets the value of the TCP_NODELAY option on this socket.
For more information about this option, see
TcpStream::set_nodelay.
Sourcepub fn set_nodelay(&self, nodelay: bool) -> Result<(), Error>
pub fn set_nodelay(&self, nodelay: bool) -> Result<(), Error>
Sets the value of the TCP_NODELAY option on this socket.
If set, this option disables the Nagle algorithm. This means that segments are always sent as soon as possible, even if there is only a small amount of data. When not set, data is buffered until there is a sufficient amount to send out, thereby avoiding the frequent sending of small packets.
Sourcepub fn linger(&self) -> Result<Option<Duration>, Error>
pub fn linger(&self) -> Result<Option<Duration>, Error>
Reads the linger duration for this socket by getting the SO_LINGER
option.
Sourcepub fn set_zero_linger(&self) -> Result<(), Error>
pub fn set_zero_linger(&self) -> Result<(), Error>
Sets a linger duration of zero on this socket by setting the SO_LINGER
option.
Sourcepub fn ttl_v4(&self) -> Result<u32, Error>
pub fn ttl_v4(&self) -> Result<u32, Error>
Gets the value of the IP_TTL option for this socket.
For more information about this option, see set_ttl_v4.
Sourcepub fn set_ttl_v4(&self, ttl: u32) -> Result<(), Error>
pub fn set_ttl_v4(&self, ttl: u32) -> Result<(), Error>
Sets the value for the IP_TTL option on this socket.
This value sets the time-to-live field that is used in every packet sent from this socket.
Sourcepub async fn send_out_of_band<T>(&self, buf: T) -> BufResult<usize, T>where
T: IoBuf,
pub async fn send_out_of_band<T>(&self, buf: T) -> BufResult<usize, T>where
T: IoBuf,
Sends out-of-band data on this socket.
Out-of-band data is sent with the MSG_OOB flag.